﻿// CSharp Editor Example with Code Completion
// Copyright (c) 2006, Daniel Grunwald
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
// 
// - Redistributions of source code must retain the above copyright notice, this list
//   of conditions and the following disclaimer.
// 
// - Redistributions in binary form must reproduce the above copyright notice, this list
//   of conditions and the following disclaimer in the documentation and/or other materials
//   provided with the distribution.
// 
// - Neither the name of the ICSharpCode team nor the names of its contributors may be used to
//   endorse or promote products derived from this software without specific prior written
//   permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS &AS IS& AND ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
// IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
// OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Threading;

using NRefactory = ICSharpCode.NRefactory;
using Dom = ICSharpCode.SharpDevelop.Dom;
using System.Diagnostics;

namespace CSharpEditor
{
	partial class MainForm
	{
		[STAThread]
		public static void Main(string[] args)
		{
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
            if ( args.Length > 0 )
                Application.Run(new MainForm(args[0]));
            else
                Application.Run(new MainForm(string.Empty));
		}
		
		internal Dom.ProjectContentRegistry pcRegistry;
		internal Dom.DefaultProjectContent myProjectContent;
		internal Dom.ICompilationUnit lastCompilationUnit;
		Thread parserThread;
        string BaseTitle = string.Empty;
		
		/// <summary>
		/// Many SharpDevelop.Dom methods take a file name, which is really just a unique identifier
		/// for a file - Dom methods don't try to access code files on disk, so the file does not have
		/// to exist.
		/// SharpDevelop itself uses internal names of the kind "[randomId]/Class1.cs" to support
		/// code-completion in unsaved files.
		/// </summary>
		public const string DummyFileName = "edited.cs";
		
		public MainForm(string file)
		{
			//
			// The InitializeComponent() call is required for Windows Forms designer support.
			//

			InitializeComponent();

            BaseTitle = this.Text;

            HandleArguments(file);

			textEditorControl1.SetHighlighting("C#");

			textEditorControl1.ShowEOLMarkers = false;
            
			CodeCompletionKeyHandler.Attach(this, textEditorControl1);

			HostCallbackImplementation.Register(this);
			
			pcRegistry = new Dom.ProjectContentRegistry(); // Default .NET 2.0 registry
			
			// Persistence caches referenced project contents for faster loading.
			// It also activates loading XML documentation files and caching them
			// for faster loading and lower memory usage.
			pcRegistry.ActivatePersistence(Path.Combine(Path.GetTempPath(),
			                                            "CSharpCodeCompletion"));
			
			myProjectContent = new Dom.DefaultProjectContent();

			myProjectContent.Language = Dom.LanguageProperties.CSharp;
            
		}

        private void SyncStatus()
        {
           string CurrentFile = textEditorControl1.FileName == null ? "New file" : textEditorControl1.FileName;

           Text = string.Format("{0} - ( {1} )", BaseTitle, CurrentFile);     
      
        
        }

        private void HandleArguments(string file)
        {
            if ( !string.IsNullOrEmpty(file) )
            {
                if ( File.Exists(file) )
                    OpenFile(file);
                else
                    NewDefaultScript();
            }
            else
                NewDefaultScript();
        }

        private void OpenFile(string file)
        {
            if ( File.Exists(file) )
                textEditorControl1.LoadFile(file, true, true);
            else
                MessageBox.Show("The file " + file + " does not exist");
        }

        private void NewFile()
        {
            textEditorControl1.Text = @"
#region IMPORTS

using System;
using System.IO;
using System.Text;
using System.Collections.Generic;

//Example of COM reference. Remove the '//#' to enable the line 
//#css_prescript com(Msxml2.DOMDocument, Msxml2); //<-- Param1: Class ID, param2: assembly name
//using Msxml2; //<-- Must be the name defined in second parameter above                

//Example on how to reference a web service: Remove the '//#' to enable the line 
//#css_prescript wsdl(http://localhost/hello/hello.asmx?WSDL, HelloService);
//#css_import HelloService;

#endregion

class MainClass
{
	static void Main(string[] args)
	{
	    Console.Write(""Hello c# script editor.."");	
	}
}
";
            textEditorControl1.FileName = null;
        }
		
		protected override void OnLoad(EventArgs e)
		{
			base.OnLoad(e);
			
			parserThread = new Thread(ParserThread);
			parserThread.IsBackground = true;
			parserThread.Start();
		}
		
		void ParserThread()
		{
			BeginInvoke(new MethodInvoker(delegate { parserThreadLabel.Text = "Loading mscorlib..."; }));
			myProjectContent.AddReferencedContent(pcRegistry.Mscorlib);
			
			// do one initial parser step to enable code-completion while other
			// references are loading
			ParseStep();
			
			string[] referencedAssemblies = {
				"System", "System.Data", "System.Drawing", "System.Xml", "System.Windows.Forms"
			};
			foreach (string assemblyName in referencedAssemblies) {
				{ // block for anonymous method
					string assemblyNameCopy = assemblyName;
					BeginInvoke(new MethodInvoker(delegate { parserThreadLabel.Text = "Loading " + assemblyNameCopy + "..."; }));
				}
				myProjectContent.AddReferencedContent(pcRegistry.GetProjectContentForReference(assemblyName, assemblyName));
			}
			BeginInvoke(new MethodInvoker(delegate { parserThreadLabel.Text = "Ready"; }));
			
			// Parse the current file every 2 seconds
			while (!IsDisposed) {
				ParseStep();
				
				Thread.Sleep(2000);
			}
		}
		
		void ParseStep()
		{
            try
            {
			    string code = null;
			    Invoke(new MethodInvoker(delegate {
			                         	    code = textEditorControl1.Text;
			                             }));
			    TextReader textReader = new StringReader(code);
			    Dom.ICompilationUnit newCompilationUnit;
			    using (NRefactory.IParser p = NRefactory.ParserFactory.CreateParser(NRefactory.SupportedLanguage.CSharp, textReader)) {
				    p.Parse();
				    newCompilationUnit = ConvertCompilationUnit(p.CompilationUnit);
			    }
			    // Remove information from lastCompilationUnit and add information from newCompilationUnit.
			    myProjectContent.UpdateCompilationUnit(lastCompilationUnit, newCompilationUnit, DummyFileName);
			    lastCompilationUnit = newCompilationUnit;
            }catch(ObjectDisposedException x)
            {

            }
		}
		
		Dom.ICompilationUnit ConvertCompilationUnit(NRefactory.Ast.CompilationUnit cu)
		{
			Dom.NRefactoryResolver.NRefactoryASTConvertVisitor converter;
			converter = new Dom.NRefactoryResolver.NRefactoryASTConvertVisitor(myProjectContent);
			cu.AcceptVisitor(converter, null);
			return converter.Cu;
		}

        private void kryptonBorderEdge1_Paint(object sender, PaintEventArgs e)
        {

        }

        private void textEditorControl1_Load(object sender, EventArgs e)
        {

        }

        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        private void kryptonButton4_Click(object sender, EventArgs e)
        {

        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Open();
        }

        private void Open()
        {
            OpenFileDialog Ofd = new OpenFileDialog();

            Ofd.CheckFileExists = true;

            Ofd.DefaultExt = ".cs";

            Ofd.Multiselect = false;

            if ( Ofd.ShowDialog() == DialogResult.OK )
                OpenFile(Ofd.FileName);

            SyncStatus();
        }

        private string GetCurrentFile()
        {
            string ActiveFile = textEditorControl1.FileName;

            if( File.Exists(ActiveFile) )
            {
                return ActiveFile;
            }
            else
            {
                return SaveAs();
            }

        }

        private string SaveAs()
        {
            SaveFileDialog Sfd = new SaveFileDialog();

            Sfd.AddExtension = true; ;

            Sfd.CheckPathExists = true;

            Sfd.DefaultExt = ".cs";

            Sfd.InitialDirectory = GetDefaultSaveDirectory();

            Sfd.ValidateNames = true;

            if ( Sfd.ShowDialog() == DialogResult.OK )
                return Sfd.FileName;
            else
                return string.Empty;
        }

        private string GetDefaultSaveDirectory()
        {
            return Path.GetDirectoryName(Application.ExecutablePath);
        }

        private string GetFileContent()
        {
            return textEditorControl1.Text;
        }

        private void WriteContent(string file, string content)
        {
            textEditorControl1.SaveFile(file);

            /*using ( StreamWriter sw = new StreamWriter(file, true) )
            {
                sw.Write(content);
            }*/
            
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Save();
        }

        private void Save()
        {
            WriteContent(GetCurrentFile(), GetFileContent());

            SyncStatus();
        }

        private void defaultScriptToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewDefaultScript();
        }

        private void NewDefaultScript()
        {
            NewFile();

            SyncStatus();

        }

        private string CombinePath(string file)
        {
            return Path.Combine( Path.GetDirectoryName(Application.ExecutablePath) , file);
        }

        private void onlineHelpToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenDefaultHelp();
        }

        private void OpenDefaultHelp()
        {
            System.Diagnostics.Process.Start("http://www.members.optusnet.com.au/~olegshilo/Documentation.html");
        }

        private void kryptonButton1_Click(object sender, EventArgs e)
        {
            RunCurrent();
        }

        private void RunCurrent()
        {
            Save();

            Debug.Write("\""+CombinePath("css.exe")+"\"");
            Debug.Write(" ");
            Debug.Write("\"" + textEditorControl1.FileName + "\"");

            System.Diagnostics.Process.Start("\""+CombinePath("css.exe")+"\"", "/nl \"" + textEditorControl1.FileName + "\"");
        }

        private void kryptonButton3_Click(object sender, EventArgs e)
        {
            DebugCurrent();
        }

        private void DebugCurrent()
        {
            Save();

            System.Diagnostics.Process.Start("\"" + CombinePath("cscs.exe") + "\"", "\"" + CombinePath("debugCLR.cs") + "\" \"" + textEditorControl1.FileName + "\"");
        }

        private void kryptonButton2_Click(object sender, EventArgs e)
        {
            Verify();
        }

        private void Verify()
        {
            Save();

            System.Diagnostics.Process.Start("\"" + CombinePath("cscs.exe") + "\"", "\"" + CombinePath("verify2.cs") + "\" \"" + textEditorControl1.FileName + "\"");
        }

        private void MainForm_KeyDown(object sender, KeyEventArgs e)
        {
            bool Ctrl = e.Control;

            #region autogenerated

            switch ( e.KeyCode )
            {
                case Keys.F5:
                    if ( Ctrl )
                        Verify();
                    else
                        RunCurrent();
                    break;
                case Keys.F10:
                    DebugCurrent();
                    break;
                case Keys.F11:
                    DebugCurrent();
                    break;
                default:
                    break;
            }

            #endregion
        }

        private void runToolStripMenuItem_Click(object sender, EventArgs e)
        {
            RunCurrent();
        }

        private void startDebuggingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DebugCurrent();
        }

        private void checkSyntaxToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Verify();
        }
	}
}
